;
; Test_SPI.asm
;
; Created: 02.11.2020 00:36:57
; Author : N0082
;
.include	"port_io.inc"
.include	"macros.inc"



.equ	F_CPU				= 16000000

#define	EXT_RESET
.equ	RST_PORT			= PORTB
.equ	RST_PIN				= 1

; OLED Display settings
.equ	Display_Xsize					= 128	// in Pixel	(DOT's)	[1..320]
.equ	Display_Ysize					= 128	// in Pixel (DOT's)	[1..256]

#define	OLED_DISPLAY_ANGEL				0		// 0 , 90 , 180 , 270 degree


;#define		USE_DELAY

.def	ZERO				= r13
.def	ONE					= r14
.def	iSREG				= r15
.def	iTEMPA				= r23

.include	"sram_variables.inc"
.include	"tn1614_INT_vectors.inc"
.include	"delay.inc"
.include	"spi.inc"
.include	"SSD1351_driver.inc"
.include	"gfx128.inc"
.include	"txt_gfx.inc"
RESET:

		clr		ZERO
		ldi		r16,1
		mov		ONE,r16

		ldi		YL,Byte1(1300)
		ldi		YH,Byte2(1300)
		load_p	Z,(INTERNAL_SRAM_START)
flush_sRAM:
		st		Z+,zero
		sbiw	Y,1
		brne	flush_sRAM


		sbi		VPORTA+0,PA7
		sbi		VPORTA+1,PA7


;---- CLK Source -----------------------------
		ldi		r16,(CLKCTRL_CLKSEL_OSC20M_gc);(1<<CLKCTRL_CLKOUT_bp)+
		ldi		r17,(CPU_CCP_IOREG_gc)
		out		(CPU_CCP),r17
		sts		(CLKCTRL_MCLKCTRLA),r16

wait_clk_osc20_stable:
		lds		r16,(CLKCTRL_MCLKSTATUS)
		sbrs	r16,(CLKCTRL_OSC20MS_bp)
		rjmp	wait_clk_osc20_stable

;---- CLK Prescaller -----------------------------
		ldi		r16,(CLKCTRL_PDIV_2X_gc)+(0<<CLKCTRL_PEN_bp)
		ldi		r17,(CPU_CCP_IOREG_gc)
		out		(CPU_CCP),r17
		sts		(CLKCTRL_MCLKCTRLB),r16

#if defined EXT_RESET
;---- RESET PIN ----------------------------------
		ldi		r16,(1<<PORT_PULLUPEN_bp)|(PORT_ISC_FALLING_gc)
		sts		((RST_PORT)+(RST_PIN)+(PORT_PIN0CTRL_offset)),r16			;PORTB_PIN1CTRL
#endif
		rcall	init_spi
		sei

		rcall	oled_init

/*
		ldi16	r16,r17,(WHITE)
		ldi		XL,10
		ldi		YL,10
		ldi		XH,4
		rcall	oled_fill_circle
*/
		rjmp	test_txt	

test_txt:

		draw_fcircle	10,10,4,(WHITE)
		draw_rect		0,0,128,128,(WHITE)
		draw_frdrect	31,109,66,13,4,((2<<RED_bp)|(16<<GREEN_bp)|(10<<BLUE_bp))
		draw_rdrect		30,108,68,15,5,(WHITE)


		set_xypos	24,24
		set_txt_color	(RED)
		set_back_color	(BLACK)
		fld_txt	(apetxt_01)							// Hallo!
		rcall	oled_write_ape_fstring


		set_xypos		52,56			;52
		set_txt_color	(BLUE)
		set_back_color	(BLACK)
		fld_txt	(apetxt_02)							// AVR
		rcall	oled_write_ape_fstring


		set_xypos		50,85
		set_txt_color	(YELLOW)
		set_back_color	(BLACK)
		fld_txt	(apetxt_03)							// ATtiny1614
		rcall	oled_write_ape_fstring

	
		set_xypos		35,112
		set_txt_color	(YELLOW)
		set_back_color	((2<<RED_bp)|(16<<GREEN_bp)|(10<<BLUE_bp))
		fld_txt	(apetxt_04)
		rcall	oled_write_ape_fstring


		lds		r16,(CharPosX)
		subi	r16,(-5)
		sts		(CharPosX),r16
		set_txt_color	(GREEN) | (10<<BLUE_bp)
		set_back_color	((2<<RED_bp)|(16<<GREEN_bp)|(10<<BLUE_bp))
		fld_txt	(apetxt_05)
		rcall	oled_write_ape_fstring


		rjmp	start


test_rotate:
		delay	2000
;		rcall	oled_clr_screen

		lds		r0,(ORIANTATION)
		sbrc	r0,3
		rjmp	reload_rotation
		clc
load_rotation:
		rol		r0
		sts		(ORIANTATION),r0
		rcall	oled_set_oriantation
		rjmp	test_rotate
reload_rotation:
		clr		r0
		sec
		rjmp	load_rotation


; Replace with your application code
start:
		inc		r16
		rjmp	start



;.include	"E:\Atmel\APE_Fonts\Arial_10x16.inc"
.include	"E:\Atmel\APE_Fonts\Comic_Sans_10x32_Bold.inc"
.include	"E:\Atmel\APE_Fonts\FONT_5x8.inc"

; Example to write STRINGS from FLASH
; LABEL:			LABEL of Font					  .db <CharSpace>,"STRING",0 <-- Termierung
;otxt_test:		.dw	(COMIC_SANS_10X32_BOLD <<1)			\ .db	1,	"Test",0

apetxt_01:		.dw	(COMIC_SANS_10X32_BOLD <<1)			\ .db	2,	"Hallo!",0
apetxt_02:		.dw	(COMIC_SANS_10X32_BOLD <<1)			\ .db	1,	"AVR",0
apetxt_03:		.dw	(FONT_5x8 <<1)						\ .db	3,	"ATtiny1614",0
apetxt_04:		.dw	(FONT_5x8 <<1)						\ .db	1,	"Mini",0
apetxt_05:		.dw	(FONT_5x8 <<1)						\ .db	1,	"Schrift",0