/*
 * spi.inc
 *
 *  Created: 20.10.2020 13:44:06
 *   Author: Steffen Hörhold
 */

;--- SPI PORT ------------------------------------------------------------------------------------------------------------
.equ	SPI0_PORT		= PORTA
.equ	SPI0_SDO		= PA1		// MOSI
.equ	SPI0_SDI		= PA2		// MISO
.equ	SPI0_SCK		= PA3		// SCK
.equ	SPI0_CS			= PA4		// /SS


 ;--- SPI Schnittstelle -----------------------------------------------------------------------------------------------------
.equ	SPI_MASTER_MODE	= 1	// 0= SPI in NORMAL Mode | 1= SPI in MASTER Mode
.equ	SPI_MODE		= 0 // [0..3]
.equ	SPI_DATA_ORDER	= 0 // 0= MSB first | 1= LSB first
.equ	SPI_DATA_RATE	= 0	// 0= F_CPU/2 | 1= F_CPU/4 | 2= F_CPU/8 | 3= F_CPU/16 | 4= F_CPU/32 | 5= F_CPU/64 | 6= F_CPU/128


.if SPI_MASTER_MODE == 0
.message "Use SPI in NORMAL Mode"
.set	SPI_MASTER = (0<<SPI_MASTER_bp)
.elif SPI_MASTER_MODE == 1
.message "Use SPI in MASTER Mode"
.set	SPI_MASTER = (1<<SPI_MASTER_bp)
.else
.error "NOT SUPPORTED OPTION FOR SPI_MASTER! PLEASE USE [0 or 1]"
.endif

.if SPI_MODE == 0
.message "Use SPI in MODE 0"
.elif SPI_MODE == 1
.message "Use SPI in MODE 1"
.elif SPI_MODE == 2
.message "Use SPI in MODE 2"
.elif SPI_MODE == 3
.message "Use SPI in MODE 3"
.else
.error "NOT SUPPORTED OPTION FOR SPI_MODE! PLEASE USE [0..3]"
.endif

.if SPI_DATA_ORDER == 0
.message "The MSB of SPI data word is trasmit first"
.set	SPI_DORD = (0<<SPI_DORD_bp)
.elif SPI_DATA_ORDER == 1
.message "The LSB of SPI data word is trasmit first"
.set	SPI_DORD = (1<<SPI_DORD_bp)
.else
.error "NOT SUPPORTED OPTION FOR SPI_DATA_ORDER! PLEASE USE [0 or 1]"
.endif

.if SPI_DATA_RATE == 0
.message "SPI DATA_RATE = F_cpu/2"
.set	SPI_CLK2X = (1<<SPI_CLK2X_bp)
.set	SPI_PRESC = SPI_PRESC_DIV4_gc
.elif SPI_DATA_RATE == 1
.message "SPI DATA_RATE = F_cpu/4"
.set	SPI_CLK2X = (0<<SPI_CLK2X_bp)
.set	SPI_PRESC = SPI_PRESC_DIV4_gc
.elif SPI_DATA_RATE == 2
.message "SPI DATA_RATE = F_cpu/8"
.set	SPI_CLK2X = (1<<SPI_CLK2X_bp)
.set	SPI_PRESC = SPI_PRESC_DIV16_gc
.elif SPI_DATA_RATE == 3
.message "SPI DATA_RATE = F_cpu/16"
.set	SPI_CLK2X = (0<<SPI_CLK2X_bp)
.set	SPI_PRESC = SPI_PRESC_DIV16_gc
.elif SPI_DATA_RATE == 4
.message "SPI DATA_RATE = F_cpu/32"
.set	SPI_CLK2X = (1<<SPI_CLK2X_bp)
.set	SPI_PRESC = SPI_PRESC_DIV64_gc
.elif SPI_DATA_RATE == 5
.message "SPI DATA_RATE = F_cpu/64"
.set	SPI_CLK2X = (0<<SPI_CLK2X_bp)
.set	SPI_PRESC = SPI_PRESC_DIV64_gc
.elif SPI_DATA_RATE == 6
.message "SPI DATA_RATE = F_cpu/128"
.set	SPI_CLK2X = (0<<SPI_CLK2X_bp)
.set	SPI_PRESC = SPI_PRESC_DIV128_gc
.else
.error "NOT SUPPORTED OPTION FOR SPI_DATA_RATE! PLEASE USE [0..6]"
.endif

;--- Macros ------------------------------------------------------------------------------------------------------------

//! Marco :Pull down the chip select line of the serial memory
.macro	spi_select_CS
	push	r16
	ldi		r16, (1<<SPI0_CS)
	sts		((SPI0_PORT)+6), r16		; PORT.OUT.CLR
	pop		r16
.endmacro

//! Macro Pull high the chip select line of the serial memory
.macro	spi_diselect_CS
	push	r16
	ldi		r16, (1<<SPI0_CS)
	sts		((SPI0_PORT)+5), r16		; PORT.OUT.SET
	pop		r16
.endmacro


init_spi:
;--- SPI-Pins initialisieren ------------------------------------
		ldi		r16,(1<<SPI0_CS)|(1<<SPI0_SCK)|(1<<SPI0_SDO)
		sts		((SPI0_PORT)+1),r16					; PORT.DIR.SET
		ldi		r16,(1<<SPI0_CS)
		sts		((SPI0_PORT)+5),r16					; PORT.OUT.SET
		ldi		r16,(1<<SPI0_SDI)
		sts		((SPI0_PORT)+2),r16					; PORT.DIR.CLR
		ldi		r16,(1<<SPI0_SCK)|(1<<SPI0_SDO)
		sts		((SPI0_PORT)+6),r16					; PORT.OUT.CLR
;--- SPI-Register CTRLA initialisieren --------------------------
		ldi		r16,(SPI_DORD)|(SPI_MASTER)|(SPI_CLK2X)|(SPI_PRESC)
		sts		(SPI0_CTRLA),r16		
;--- SPI-Register CTRLB initialisieren --------------------------
		ldi		r16,(1<<SPI_BUFEN_bp)|(1<<SPI_SSD_bp)|(SPI_MODE)		; Buffer EN
		sts		(SPI0_CTRLB),r16
;--- Enable TXCready Interrupt
/*		ldi		r16,(1<<SPI_TXCIE_bp)
		sts		(SPI_INTCTRL),r16			*/
		rcall	spi_enable
		ret

spi_enable:
		lds		r16,(SPI0_CTRLA)
		sbr		r16,(1<<SPI_ENABLE_bp)
		sts		(SPI0_CTRLA),r16
		ret

spi_disable:
		lds		r16,(SPI0_CTRLA)
		cbr		r16,(1<<SPI_ENABLE_bp)
		sts		(SPI0_CTRLA),r16
		ret

spi_write:
		lds		r0,(SPI0_INTFLAGS)
		sbrs	r0,(SPI_DREIE_bp)
		rjmp	spi_write
		sts		(SPI0_DATA),r16
		ret

wait_spi_write_done:
		lds		r0,(SPI0_INTFLAGS)
		sbrs	r0,(SPI_TXCIF_bp)
		rjmp	wait_spi_write_done
		sts		(SPI0_INTFLAGS),r0
		ret

spi_transfer16:
		lds		r0,(SPI0_INTFLAGS)
		sbrs	r0,(SPI_DREIE_bp)
		rjmp	spi_transfer16
		sts		(SPI0_DATA),r16
		lds		r0,(SPI0_INTFLAGS)
		sbrs	r0,(SPI_DREIE_bp)
		rjmp	spi_transfer16
		sts		(SPI0_DATA),r17
wait_spi_transfer16_done:
		lds		r0,(SPI0_INTFLAGS)
		sbrs	r0,(SPI_TXCIF_bp)
		rjmp	wait_spi_transfer16_done
		lds		r16,(SPI0_DATA)
		lds		r17,(SPI0_DATA)
		sts		(SPI0_INTFLAGS),r0
		ret

SPI0_INT_INT:
		